Package: panelr
Title: Regression Models and Utilities for Repeated Measures and Panel
        Data
Version: 0.7.8
Authors@R: person("Jacob A.", "Long", email = "jacob.long@sc.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-1582-6214"))
Description: Provides an object type and associated tools for storing and 
  wrangling panel data. Implements several methods for creating regression
  models that take advantage of the unique aspects of 
  panel data. Among other capabilities, automates the "within-between" 
  (also known as "between-within" and "hybrid") panel regression specification
  that combines the desirable aspects of both fixed effects and random effects 
  econometric models and fits them as multilevel models 
  (Allison, 2009 <doi:10.4135/9781412993869.d33>; 
  Bell & Jones, 2015 <doi:10.1017/psrm.2014.7>). These models can also be 
  estimated via generalized estimating equations 
  (GEE; McNeish, 2019 <doi:10.1080/00273171.2019.1602504>) and Bayesian 
  estimation is (optionally) supported via 'Stan'. 
  Supports estimation of asymmetric effects models via first differences
  (Allison, 2019 <doi:10.1177/2378023119826441>) as well as a generalized
  linear model extension thereof using GEE. 
URL: https://panelr.jacob-long.com
BugReports: https://github.com/jacob-long/panelr
Depends: R (>= 3.4.0), lme4
Imports: crayon, dplyr, Formula, ggplot2, jtools (>= 2.0.1), lmerTest,
        magrittr, methods, purrr, rlang (>= 0.3.0), stringr, tibble (>=
        2.0.0), tidyr (>= 0.8.3)
Suggests: brms, broom.mixed, car, clubSandwich, geepack, generics,
        nlme, plm, sandwich, skimr, testthat, covr, knitr, rmarkdown
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-08-21 16:59:59 UTC; jacoblong
Author: Jacob A. Long [aut, cre] (<https://orcid.org/0000-0002-1582-6214>)
Maintainer: Jacob A. Long <jacob.long@sc.edu>
Repository: CRAN
Date/Publication: 2023-08-22 15:50:08 UTC
Built: R 4.4.0; ; 2024-06-02 22:55:18 UTC; unix
