# Generated by roxygen2: do not edit by hand


if (getRversion() >= "3.6.0") {
  S3method(dplyr::filter, panel_data)
} else {
  export(filter.panel_data)
}

if (getRversion() >= "3.6.0") {
  S3method(dplyr::select, panel_data)
} else {
  export(select.panel_data)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::glance, fdm)
} else {
  export(glance.fdm)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::glance, summ.wbm)
} else {
  export(glance.summ.wbm)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::glance, wbgee)
} else {
  export(glance.wbgee)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::glance, wbm)
} else {
  export(glance.wbm)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::tidy, asym)
} else {
  export(tidy.asym)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::tidy, asym_gee)
} else {
  export(tidy.asym_gee)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::tidy, fdm)
} else {
  export(tidy.fdm)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::tidy, summ.wbm)
} else {
  export(tidy.summ.wbm)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::tidy, wbgee)
} else {
  export(tidy.wbgee)
}

if (getRversion() >= "3.6.0") {
  S3method(generics::tidy, wbm)
} else {
  export(tidy.wbm)
}

if (getRversion() >= "3.6.0") {
  S3method(jtools::summ, wbm)
} else {
  export(summ.wbm)
}

if (getRversion() >= "3.6.0") {
  S3method(knitr::knit_print, summary.panel_data)
} else {
  export(knit_print.summary.panel_data)
}
S3method("[",panel_data)
S3method("[<-",panel_data)
S3method("[[<-",panel_data)
S3method("names<-",panel_data)
S3method(anti_join,panel_data)
S3method(arrange,panel_data)
S3method(arrange_,panel_data)
S3method(as_panel_data,default)
S3method(as_panel_data,pdata.frame)
S3method(coef,fdm)
S3method(confint,fdm)
S3method(confint,wbgee)
S3method(df.residual,wbm)
S3method(distinct,panel_data)
S3method(formula,wbgee)
S3method(formula,wbm)
S3method(full_join,panel_data)
S3method(getCall,wbm)
S3method(group_by,panel_data)
S3method(inner_join,panel_data)
S3method(left_join,panel_data)
S3method(make_predictions,wbgee)
S3method(make_predictions,wbm)
S3method(mutate,panel_data)
S3method(mutate_,panel_data)
S3method(nest,panel_data)
S3method(nobs,wbm)
S3method(partialize,wbm)
S3method(predict,wbgee)
S3method(predict,wbm)
S3method(print,asym_gee)
S3method(print,summary.asym)
S3method(print,summary.asym_gee)
S3method(print,summary.fdm)
S3method(print,summary.panel_data)
S3method(print,summary.wbgee)
S3method(print,summary.wbm)
S3method(print,wbgee)
S3method(print,wbm)
S3method(reconstruct,panel_data)
S3method(right_join,panel_data)
S3method(semi_join,panel_data)
S3method(simulate,wbm)
S3method(slice,panel_data)
S3method(slice_,panel_data)
S3method(summarise,panel_data)
S3method(summarise_,panel_data)
S3method(summarize_,panel_data)
S3method(summary,asym)
S3method(summary,asym_gee)
S3method(summary,fdm)
S3method(summary,panel_data)
S3method(summary,wbgee)
S3method(summary,wbm)
S3method(summary,wbm_stan)
S3method(tbl_sum,panel_data)
S3method(terms,wbm)
S3method(transmute,panel_data)
S3method(vcov,fdm)
S3method(vcov,wbgee)
export("%<>%")
export("%>%")
export(are_varying)
export(as_panel)
export(as_panel_data)
export(as_pdata.frame)
export(asym)
export(asym_gee)
export(complete_data)
export(fdm)
export(filter)
export(get_id)
export(get_periods)
export(get_wave)
export(heise)
export(is_panel)
export(line_plot)
export(long_panel)
export(make_diff_data)
export(make_wb_data)
export(model_frame)
export(panel_data)
export(unpanel)
export(wbgee)
export(wbm)
export(wbm_stan)
export(widen_panel)
exportClasses(wbm)
import(dplyr)
import(jtools)
import(rlang)
import(stringr)
importFrom(crayon,bold)
importFrom(crayon,italic)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,arrange_)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,slice)
importFrom(dplyr,slice_)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_)
importFrom(dplyr,summarize_)
importFrom(dplyr,transmute)
importFrom(ggplot2,aes)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(jtools,make_predictions)
importFrom(lme4,isLMM)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(methods,is)
importFrom(purrr,map_lgl)
importFrom(rlang,":=")
importFrom(rlang,UQ)
importFrom(rlang,UQS)
importFrom(rlang,syms)
importFrom(stats,"contrasts<-")
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,contr.treatment)
importFrom(stats,cor)
importFrom(stats,delete.response)
importFrom(stats,df.residual)
importFrom(stats,drop.terms)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,getCall)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,na.action)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,nobs)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,reshape)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(tibble,deframe)
importFrom(tibble,tbl_sum)
importFrom(tidyr,nest)
importFrom(utils,getFromNamespace)
